import {
  ChangeDetectionStrategy,
  Component,
  ComponentFactoryResolver,
  Input,
  OnInit,
  ViewChild,
} from '@angular/core';

import { DynamicHostDirective } from '../../directives';
import { DynamicComponentData } from '../../models';

@Component({
  selector: 'm3-dynamic-component',
  templateUrl: './dynamic.component.html',
  styleUrls: ['./dynamic.component.css'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DynamicComponent implements OnInit {
  @ViewChild(DynamicHostDirective, { static: true })
  componentHost: DynamicHostDirective;

  @Input() set componentData(d: DynamicComponentData) {
    if (this.noDataOrHostContainerRef(d)) {
      return;
    }
    this._componentData = d;
    this.createComponent();
  }
  get componentData(): DynamicComponentData {
    return this._componentData;
  }
  private _componentData: DynamicComponentData;

  constructor(private cfr: ComponentFactoryResolver) {}

  ngOnInit(): void {
    console.log('Dynamic host:', this.componentHost);
  }

  private createComponent(): void {
    const componentFactory = this.cfr.resolveComponentFactory(
      this.componentData.component
    );

    const containerRef = this.componentHost.viewContainerRef;
    containerRef.clear();

    const componentRef = containerRef.createComponent(componentFactory);

    Object.keys(this.componentData.inputs).forEach((key) => {
      componentRef.instance[key] = this.componentData.inputs[key];
    });
  }

  private noDataOrHostContainerRef(data: DynamicComponentData): boolean {
    return !data || !this.componentHost?.viewContainerRef;
  }
}
