import { ChangeDetectionStrategy, Component, ViewChild } from '@angular/core';

import { BehaviorSubject } from 'rxjs';
import { shareReplay } from 'rxjs/operators';
import { SohoTabsComponent } from 'ids-enterprise-ng';

import { TabItem } from '../../models';
import { IframeComponent } from '../iframe/iframe.component';
import { DisplayTextComponent } from '../display-text/display-text.component';

@Component({
  selector: 'm3-tabs',
  templateUrl: './tabs.component.html',
  styleUrls: ['./tabs.component.css'],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class TabsComponent {
  @ViewChild(SohoTabsComponent, { static: true })
  sohoTabsComponent: SohoTabsComponent;

  private tabItems: TabItem[] = [];
  private tabLisItems = new BehaviorSubject<TabItem[]>([]);
  tabItems$ = this.tabLisItems.asObservable().pipe(shareReplay());

  constructor() {}
  onBeforeClose = (event: SohoTabsEvent, tabLi: JQuery): void => {
    // would be nice to have a better way...
    const id = tabLi.find('a').attr('href').split('#')[1];
    this.removeTab(id);
  };

  addDisplayTextTab(): void {
    const id = this.generateId();
    const tabItem: TabItem = {
      id,
      title: `Display Text Tab: ${id}`,
      data: {
        component: DisplayTextComponent,
        inputs: {
          text: 'This simple text is important...',
        },
      },
    };

    this.updateTabs(tabItem);
  }

  addIframeTab(): void {
    const id = this.generateId();
    const tabItem: TabItem = {
      id,
      title: `Iframe tab: ${id}`,
      data: {
        component: IframeComponent,
        inputs: {
          url: 'https://www.google.com/webhp?igu=1',
        },
      },
    };
    this.updateTabs(tabItem);
  }

  trackByTabId(index: number, { id }: TabItem): string {
    return id;
  }

  onTabActivated({ tab }: SohoTabsEvent) {
    const tabId = tab.getAttribute('href')[1];
  }

  onAfterClose(event: any): void {
    console.log('onAfterClose', this.tabItems);
  }

  private updateTabs(tabItem: TabItem): void {
    this.tabItems = [...this.tabItems, tabItem];
    this.tabLisItems.next(this.tabItems);
  }

  private removeTab(id: string): void {
    this.tabItems = this.tabItems.filter((tab) => tab.id !== id);
    this.tabLisItems.next(this.tabItems);
  }

  private generateId(): string {
    return '_' + Math.random().toString(36).substr(2, 9);
  }
}
